#!/usr/bin/env python
#
# applet.py - A GNOME applet that shows what Moosic is currently playing.
#
# Copyright (C) 2003 Daniel Pearson <daniel.pearson@umbc.edu>
# Copyright (C) 2003 Paramjit Oberoi <param@cs.wisc.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk
import gnome.applet

import socket, errno, os.path

# Register Icons

import icons

# The icons.py module contains string representations of
# icons, created using the "gdk-pixbuf-csource" program.
# The namespace of the module is scanned, and all the
# available icons are registered as stock icons so that
# they can be referenced simply using the icon name.
#
# The gnome convention for icon names is to use dashes
# as in "gtk-clear" or "media-pause".  Python variables
# can't contain dashes, so icons.py contains underscores
# instead, and these are replaced with dashes when the
# icons are registered.

factory = gtk.IconFactory()
factory.add_default()
for k in icons.__dict__.keys():
    if k.startswith("__"): continue
    img = icons.__dict__[k]
    name = "moo-"+k.replace("_","-")
    pixbuf = gtk.gdk.pixbuf_new_from_inline(len(img), img, 0)
    iconset = gtk.IconSet(pixbuf)
    #print "registering icon", name
    factory.add(name, iconset)


class MoosicAppletUI:

    def paused(self):
        self.pause_toggle_icon.set_from_stock('moo-media-pause-red',gtk.ICON_SIZE_BUTTON)
        self.tooltips.set_tip(self.pause_toggle_button, 'Unpause')

    def unpaused(self):
        self.pause_toggle_icon.set_from_stock('moo-media-pause',gtk.ICON_SIZE_BUTTON)
        self.tooltips.set_tip(self.pause_toggle_button, 'Pause')

    def stopped(self):
        self.play_toggle_icon.set_from_stock('moo-media-play',gtk.ICON_SIZE_BUTTON)
        self.tooltips.set_tip(self.play_toggle_button, 'Play')

    def playing(self):
        self.play_toggle_icon.set_from_stock('moo-media-stop',gtk.ICON_SIZE_BUTTON)
        self.tooltips.set_tip(self.play_toggle_button, 'Stop')

    def update_tooltip(self, s):
        #self.tooltips.set_tip(self.app_parent, s)
        #self.tooltips.set_tip(self.applet, s)
        self.tooltips.set_tip(self.ebox, s)

    # ---- Buttons

    def create_buttons(self, hbox):
        Buttons = []
        Buttons += [(self.moo.play_toggle,  'moo-media-play',   'Play')]
        Buttons += [(self.moo.pause_toggle, 'moo-media-pause',  'Pause')]
        Buttons += [(self.moo.prev, 'moo-media-rewind', 'Previous')]
        Buttons += [(self.moo.next, 'moo-media-ffwd',   'Next')]

        for (func,icon, tip) in Buttons:
            button = gtk.Button()
            buttonicon = gtk.image_new_from_stock(icon, gtk.ICON_SIZE_BUTTON)
            button.add(buttonicon)
            # size_request(0) makes the button as small as possible
            # otherwise the button is too large with gtk-2.4
            # (but, this also prevents buttons from expanding
            # automatically if the panel is larger)
            button.set_size_request(-1,0)
            hbox.add(button)
            button.connect("clicked", lambda btn,f=func: f())
            self.__dict__[func.__name__ + "_button"] = button
            self.__dict__[func.__name__ + "_icon"] = buttonicon
            self.tooltips.set_tip(button, tip)

    # ---- Menu entries

    def create_menu(self):
        Menuitems = []
        Menuitems += [(self.moo.sort, 'Sor_t Items', 'gtk-sort-ascending')]
        Menuitems += [(self.moo.shuffle, '_Shuffle Items', 'gtk-refresh')]
        Menuitems += [(self.moo.qstop, 'S_top Queue', 'moo-media-stop')]
        Menuitems += [(self.moo.clear, 'Clear _Queue', 'gtk-clear')]
        Menuitems += [(self.moo.wipe, '_Clear Queue and Stop', 'gtk-stop')]
        Menuitems += [(None, None, None)]  # seperator

        # def toggle_loop(self, *args):
        #     print "zz"
        #     self.menu.get_property("Loop")
        # Menuitems += [(toggle_loop, 'toggle_loop', 'gtk-help')]

        Menuitems += [(self.moo.config.prefs_ui, '_Preferences', 'gtk-preferences')]

        # Add entries to the applet's popup menu.
        verbs = []
        xml_template = """<menuitem name="%(name)s" verb="%(name)s" _label="%(label)s"
                                    pixtype="stock" pixname="%(icon-name)s" />"""
        menu_xml = '<popup name="button3">'
        for (func,label,icon) in Menuitems:
            if not func:
                menu_xml += "<separator/>"
            else:
                menu_xml += xml_template % {
                                    "name" : func.__name__,
                                    "label": label,
                                    "icon-name": icon }
                verbs += [(func.__name__, lambda gob,mname,f=func: f())]

        # def loop_verb(*args):
        #     print "loop_verb called with",len(args),"arguments"
        #     for i in args: print type(i), i
        #
        # self.loop_verb = 0
        # menu_xml += '<menuitem name="Loop" verb="Loop" _label="_Loop" type="toggle" />'
        # verbs += [("Loop", self.loop_verb)]

        menu_xml += '</popup>'
        self.applet.setup_menu(menu_xml, verbs, None)


    # ---- Initialize the applet.

    # Widget tree is as follows:
    #
    #     self.applet
    #     |
    #     +--self.frame (for border)
    #        |
    #        +-- hbox
    #            |
    #            +--self.ebox (EventBox for a song-name tooltip)
    #            |  |
    #            |  +--hbox2
    #            |     |
    #            |     +--cow-logo
    #            |     |
    #            |     +--qlen_label
    #            |     |
    #            |     +--label
    #            |     |
    #            |     +--bogus-label-for-padding
    #            |
    #            +--Button1
    #            |
    #            +--Button2
    #            |
    #            +--<more buttons...>

    # Each button has a tooltip; self.ebox has a tooltip as well.
    # self.ebox exists only for the tooltip; assigning that tooltip
    # to self.applet leads to the tooltip not being displayed when
    # the mouse is moved off one of the buttons onto other areas.

    def __init__(self, applet, moo):
        self.applet = applet
        self.moo = moo
        self.frame = gtk.Frame()
        # Choose from ETCHED_IN / ETCHED_OUT / IN / OUT / NONE
        self.frame.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        applet.add(self.frame)

        hbox = gtk.HBox()
        self.frame.add(hbox)

        self.ebox = gtk.EventBox()
        hbox.add(self.ebox)
        hbox2 = gtk.HBox()
        self.ebox.add(hbox2)

        # Logo
        hbox2.add(gtk.image_new_from_stock('moo-cow',gtk.ICON_SIZE_BUTTON))

        # Queue length
        self.qlen_label = gtk.Label("")
        hbox2.add(self.qlen_label)

        # Current Song
        self.label = gtk.Label("")
        self.label.set_padding(5,0)
        self.label.set_size_request(self.moo.config.width,-1)
        hbox2.add(self.label)

        # dummy label for padding
        # (there must be a better way of doing this)
        # (the alignment widget?)
        l = gtk.Label("")
        l.set_padding(3,0)
        hbox2.add(l)

        self.tooltips = gtk.Tooltips()
        #self.app_parent = applet.get_parent()

        self.create_buttons(hbox)

        self.create_menu()

        # self.menu = applet.get_popup_component()
        # print self.menu
        # print type(self.menu)
        # print dir(self.menu)

        gtk.timeout_add(500, moo.refresh)

        def bye(*args):
            # this is needed to avoid warnings about leaked objects
            del self.applet
        applet.connect("destroy",bye)

        #pdb.set_trace()

        # How to add a handle to the applet?
        # 4 is supposed to denote the flag PANEL_APPLET_HAS_HANDLE, but it doesn't work
        # (there is also a 'handlebutton' I believe...)
        # self.applet.set_flags(4)

        applet.show_all()

